<?php

/**
 * Original module: LiveStreet
 * Reworked by SparklingFire
 */

return [

    /**
     * Производить обход всех правил на блокировки вне зависимости от факта блокировки
     */
    'processing_block_rule_all' => false,

    /**
     * Список запрещающих правил
     * Для каждого действия составляется отдельная группа правил, групп может быть несколько
     * Список действий: vote_comment, vote_topic, vote_user, create_comment, create_topic, create_wall
     * В параметре msg можно указать сообщение, которое будет отображаться пользователю, можно использовать просто текст или указать ключ из языкового файла, например, 'message_cant_vote'
     * В параметре type можно задать список типов над действием, например, для голосований это будет направление голосования - down - в минус, up - в плюс
     *
     * Список доступных критериев:
     *	user_id - ID или список ID пользователей, если текущий пользователь с этом списке, то запрещаем ему действие
     *
     * Любой из критериев можно опустить.
     * Сначала происходит проверка на запрещающие правила, и только потом проверяются правила $config['rule']
     */
    'rule_disallow' => [
        'vote_comment' => [
            'msg' => 'Вам нельзя голосовать на этом сайте!',
            'type' => ['down'],
            'groups' => [
                [
                    'user_id' => [1] // Для примера
                ]
            ]
        ]
    ],

    /**
     * Список правил
     * Для каждого действия составляется отдельная группа правил, групп может быть несколько
     * Список действий: vote_comment, vote_topic, vote_user, create_comment, create_topic, create_wall
     * В параметре msg можно указать сообщение, которое будет отображаться пользователю, можно использовать просто текст или указать ключ из языкового файла, например, 'message_cant_vote'
     *
     * Список доступных критериев:
     *	registration_time - время в секундах с момента регистрации пользователя
     *	rating - необходимый минимальный рейтинг
     *	skill - необходимая минимальная сила
     *	count_comment - необходимое минимальное количество комментариев у пользователя
     *	count_topic - необходимое минимальное количество топиков у пользователя
     *  rating_sum_topic - необходимый суммарный рейтинг топиков пользователя за период. По умолчанию период 14 дней, для изменения периода можно указать значение ввиде массива array(rating,period)
     *  rating_sum_comment - необходимый суммарный рейтинг комментариев пользователя за период. По умолчанию период 7 дней, для изменения периода можно указать значение ввиде массива array(rating,period)
     *
     * Любой из критериев можно опустить.
     */
    'rule' => [
        'vote_comment' => [
            'msg' => 'Вы ещё недостаточно окрепли!',
            'groups' => [
                [
                    'registration_time' => 60 * 60 * 24,
                    'rating' => 25
                ]
            ]
        ],

        'vote_topic' => [
            'msg' => 'Вы ещё недостаточно окрепли!',
            'groups' => [
                [
                    'registration_time' => 60 * 60 * 24,
                    'rating' => 25
                ]
            ]
        ],

        'vote_user' => [
            'msg' => 'Вы ещё недостаточно окрепли!',
            'groups' => [
                [
                    'registration_time' => 60 * 60 * 24,
                    'rating' => 25
                ]
            ]
        ]
    ],

    /**
     * Правила для создания блокировок
     * Группы правил необходимо располагать по приоритету - вверху самые главные
     * Если группа правил сработала, то создается блокировка и обход нижестоящих правил прекращается (только при $config['processing_block_rule_all']=false;)
     *
     * Параметры правил:
     *	name - название правила блокировки, для учета в таблице блокировок. Название ограничено в 50 символов
     * 	type - список типов голосований, up - за, down - против
     *	target - список сущностей за которые голосуют
     * 	count - количество голосований
     * 	rating - максимальный порог рейтинга пользователя, блокировка используется только если рейтинг пользователя ниже этого
     * 	period - период, за который необходимо считать количество голосований, в секундах
     * 	block_time - время, на которое необходимо блокировать возможность голосования
     * 	block_msg - сообщение, которое будет видеть пользователь при попытке голосования
     */
    'block_rule_vote' => [
        [
            'name' => 'block 1',
            'type' => ['down'],
            'target' => ['topic','comment'],
            'count' => 3,
            'rating' => 5,
            'period' => 60 * 10,
            'block_time' => 60 * 60 * 3,
            'block_msg' => 'Вы слишком агрессивны, нужно отдохнуть!'
        ]
    ]
];
